/**
 * Juliac
 * Copyright (C) 2009-2012 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Christophe Munilla
 */
package org.objectweb.fractal.juliac.osgi;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.objectweb.fractal.juliac.osgi.concierge.Concierge;
import org.objectweb.fractal.juliac.osgi.revision.ConciergeRevision;
import org.objectweb.fractal.juliac.osgi.revision.OSGiRevisionException;
import org.objectweb.fractal.juliac.osgi.revision.api.BundleContextRevisionItf;
import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleException;

/**
 * Helper methods for dealing with OSGi/Concierge.
 * */
public class PlatformImpl implements PlatformItf
{

    private ConciergeRevision r3Revision = null;


    /** 
     * {@inheritDoc}
     * 
     * @see org.objectweb.fractal.juliac.osgi.PlatformItfRevision#
     * getBundleContext()
     */
    public BundleContextRevisionItf<?, ?> getBundleContext() throws OSGiRevisionException
    {
        if (r3Revision == null)
        {        
            try
            {                
                BundleContext bundleContext = getConcierge().getBundleContext();
                r3Revision = new ConciergeRevision(bundleContext);     

            } catch (IOException e)
            {
                e.printStackTrace();

            } catch (BundleException e)
            {
                e.printStackTrace();
            }
        }
        return r3Revision.getBundleContext();
    }
    
    /**
     * Stop the platform.
     */
    public void stop() throws OSGiRevisionException
    {
        try
        {
            getConcierge().stop();
            
        } catch (IOException e)
        {
            e.printStackTrace();
            throw new OSGiRevisionException(e.getMessage());

        } catch (BundleException e)
        {
            e.printStackTrace();
            throw new OSGiRevisionException(e.getMessage());
        }
    }

    // ---------------------------------------------------------------------
    // Implementation specific
    // ---------------------------------------------------------------------

    /** The instance of the OSGi Concierge Framework */
    private static Concierge concierge;

    /**
     * Return the singleton instance of the OSGi Concierge framework.
     */
    private static Concierge getConcierge() throws IOException, BundleException
    {
        if (concierge == null)
        {
            /* Prepare a cache directory for Concierge. */
            File tmpFile = null;
            tmpFile = File.createTempFile("concierge", ".tmp");
            tmpFile.delete();
            
            // Retrieve the directory containing temporary files
            File dir = tmpFile.getParentFile();
            File cache = new File(dir, "concierge");
            
            // Delete files which may have been stored by a previous run
            deleteAllRecursively(cache);
            cache.mkdir();
            System.setProperty("ch.ethz.iks.concierge.decompressEmbedded", "false");
            InputStream is = Thread.currentThread().getContextClassLoader().getResourceAsStream(
                    "init.xargs");
            
            String path = cache.getAbsolutePath() + File.separator + "storage";
            new File(path).mkdir();
            path = cache.getAbsolutePath() + File.separator  + "init.xargs";
            
            if(write(is,path))
            {
              System.setProperty("xargs", path);
            }
            String autoInstallProperty  = System.getProperty("osgi.auto.start.1","");
            autoInstallProperty  = new StringBuilder(
                    "shell-1.0.0.jar service-tracker-1.0.0.jar event-admin-1.0.0.jar ").append(
                           autoInstallProperty).toString();
            
            System.setProperty("osgi.auto.start.1", autoInstallProperty);
            System.setProperty("osgi.auto.install.1"," ");

            StringTokenizer tokenizer = new StringTokenizer(autoInstallProperty);
            List<String> bundles = new ArrayList<String>();
            while(tokenizer.hasMoreElements())
            {
                bundles.add(tokenizer.nextToken());
            }
            installResources(cache,bundles);            
            concierge = Concierge.getInstance(cache);
        }
        return concierge;
    }

    private static void installResources(File cache,List<String> bundles)
    {
        String[] resources = bundles.toArray(new String[0]);
        
        InputStream is = null;
        String path = null;
        
        File dataStore = new File(cache.getAbsolutePath() + File.separator + "data");
        dataStore.mkdir();
        
        for (String resource : resources)
        {
            is = Thread.currentThread().getContextClassLoader().getResourceAsStream(resource);
            path = cache.getAbsolutePath() + File.separator + "data" + 
            File.separator + resource;
            write(is,path);
        }
    }

    private static boolean write(InputStream is,String path)
    {
        int BUFFER_SIZE = 1024 * 64;
     
        if (is == null)
        {
            return false;
            
        } else            
        {
            byte[] buffer = new byte[BUFFER_SIZE];
            File file = new File(path);
            if (file.exists())
            {
                return true;
            }
            FileOutputStream fos = null;
            try
            {
                fos = new FileOutputStream(file);
                int len;
                while ((len = is.read(buffer)) >= 0)
                {
                    fos.write(buffer, 0, len);
                }
            } catch (FileNotFoundException e)
            {
                //e.printStackTrace();
                return false;
            } catch (IOException e)
            {
                //e.printStackTrace();
                return false;
            } catch (NullPointerException e)
            {
                //e.printStackTrace();
                return false;
            }
            // close both streams.
            try
            {
                is.close();
                fos.close();
                return true;
            } catch (IOException e)
            {
                //e.printStackTrace();
            } catch (NullPointerException e)
            {
                //e.printStackTrace();
            } 
        }
        return false;
    }
    
    /**
     * Delete recursively all the files contained in the specified file if the
     * specified file is a directory, and the specified file itself.
     * 
     * @param f
     *            the file to delete
     */
    private static void deleteAllRecursively(File f)
    {
        if (f.isDirectory())
        {
            File[] subs = f.listFiles();
            for (int i = 0; i < subs.length; i++)
            {
                deleteAllRecursively(subs[i]);
            }
        }
        f.delete();
    }
}